import fs from 'fs-extra';

/**
 * Creates a new Date object with the specified hour and minute.
 *
 * @param {number} hours - The hour to set for the new Date
 * @param {number} minutes - The minute to set for the new Date
 * @return {Date} The new Date object with the specified hour and minute
 *
 * @example
 * // creates a Date object with the hour 12 and minute 30
 *
 * // if current time is 12:30
 *
 * const newDate = createDate(12, 30);
 * console.log(newDate);
 * @result Wed Dec 31 2022 12:30:00
 */
declare const createDate: (hours: number, minutes: number) => Date;
/**
 * Creates a new Date object that represents a specific date and time in the future, based on the current date and time.
 *
 * @param {number} offsetInDays - The number of days to offset from the current date and time.
 * @return {Date} - The new Date object representing the future date and time.
 */
declare const createDateFromNowWithOffset: (offsetInDays: number) => Date;

/**
 * Deletes the specified directory if it exists.
 *
 * @param {string} directoryPath - The path of the directory to be deleted
 * @return {boolean} Returns true if the directory is successfully deleted, otherwise false
 */
declare const fsDeleteDirectory: (directoryPath: string) => Promise<boolean>;
/**
 * Deletes a file at the specified filePath.
 *
 * @param {string} filePath - The path of the file to be deleted.
 * @return {Promise<boolean>} A Promise that resolves to true if the file is successfully deleted.
 */
declare const fsDeleteFile: (filePath: string) => Promise<boolean>;

type DownloadFileOptions = {
    url: string;
    fileName?: string;
    directoryPath: string;
    limit?: {
        minBytes?: number;
        maxBytes?: number;
    };
};
interface DownloadFile {
    path: string;
    size: number;
    sizeInMB: string;
    fileType: string;
    fileTypeExt: string;
    fileName: string;
    url: string;
    createdAt: Date;
}
declare const downloadFileFromUrl: ({ url, fileName, directoryPath, limit, }: DownloadFileOptions) => Promise<DownloadFile>;

type JSONTypes = Object | Array<any> | Object[] | Array<Object> | string;
/**
 * Reads a JSON file from the given file path and returns its content as a parsed JSON object.
 *
 * @param {string} filePath - The path of the JSON file to be read.
 * @return {Promise<JSONTypes>} A promise that resolves to the parsed JSON object.
 * @throws {Error} If there was an error reading the file or parsing its content.
 */
declare const readJSONFile: (filePath: string) => Promise<JSONTypes>;
/**
 * Write JSON data to a file.
 *
 * @param {string} filePath - the path to the file
 * @param {JSONTypes} jsonData - the JSON data to be written
 * @param {fs.JsonWriteOptions} [options] - the options for writing JSON
 * @return {Promise<boolean>} whether the JSON data was successfully written to the file
 */
declare const writeJsonToFile: (filePath: string, jsonData: JSONTypes, options?: fs.JsonWriteOptions) => Promise<boolean>;

/**
 * Writes the given content to a file in the specified directory with the given file name.
 *
 * @example
 * const filePath = await writeToFile('/path/to/directory', 'content', 'filename.txt');
 * console.log(filePath);
 * // '/path/to/directory/filename.txt'
 *
 * If the directory does not exist, it will be created.
 * If the file already exists, it will be overwritten.
 * If the directory or the file cannot be created, an error will be thrown.
 *
 * @param {string} directory - The directory where the file will be written.
 * @param {string | Buffer | Uint8Array} content - The content to be written to the file.
 * @param {string} fileName - The name of the file.
 * @return {Promise<string>} - A promise that resolves to the file path where the content was written.
 */
declare const writeToFile: (directory: string, content: string | Buffer | Uint8Array, fileName: string) => Promise<string>;

declare const readFile: (filePath: string) => Promise<string | Buffer>;
declare const getFile: (filePath: string) => Promise<string | Buffer>;

/**
 * Generates a random string of the specified length.
 *
 * @param length - The length of the string to be generated. Must be a non-negative integer.
 * @throws {Error} Throws an error if length is not a non-negative integer.
 * @returns A string of random characters of the specified length.
 */
declare const randomStringOfLength: (length: number) => string;

/**
 * Generates a random number between the specified minimum and maximum values (inclusive).
 *
 * @param min - The minimum value of the range. Must be a number.
 * @param max - The maximum value of the range. Must be a number.
 * @returns A random number between min and max (inclusive).
 * @throws {TypeError} if min or max is not a number.
 * @throws {RangeError} if min is greater than max.
 */
declare const generateRandomNumberBetween: (min: number, max: number) => number;

type OpenSslMethod = "base64" | "hex" | "binary";
/**
 * Generates a random key using OpenSSL.
 *
 * @param length The length of the random key in bytes. Must be a positive number.
 * @param method The method used to generate the random key. Defaults to "base64".
 * @returns The generated random key as a string.
 * @throws {Error} Throws an error if the length is not a positive number or if the OpenSSL command execution fails.
 */
declare const genOpensslRand: (length: number, method?: OpenSslMethod) => string;

/**
 * Checks if a given string is a valid IPv4 address.
 *
 * @param {string|null|undefined} ipAddress - The IP address to validate.
 * @returns {boolean} True if the IP address is valid, false otherwise.
 * @throws {TypeError} Throws a TypeError if ipAddress is not a string.
 */
declare const isValidIpAddress: (ipAddress: string | null | undefined) => boolean;

/**
 * Checks if a given string is a valid JSON.
 *
 * @param {string} json - The string to be checked.
 * @returns {boolean} - Returns `true` if the string is valid JSON, `false` otherwise.
 */
declare const isValidJson: (json: string) => boolean;

/**
 * Check if a given string is a valid URL.
 * @param url The string to be checked.
 * @param allowedProtocols An optional array of allowed protocols.
 * @returns {boolean} Returns true if the url is a valid URL, false otherwise.
 * @throws {TypeError} Throws a TypeError if url is not a string.
 */
declare const isValidUrl: (url: string, allowedProtocols?: string[]) => boolean;

declare const bytesToMb: (bytes: number) => number;

/**
 * Convert an epoch time (in seconds) to a Date object.
 *
 * @param epoch - The epoch time to convert.
 * @returns {Date} - The converted Date object.
 * @throws {Error} - If epoch is not a valid number.
 */
declare const epochToDate: (epoch: number) => Date;

type Props = {
    gitHost?: string;
    user: string;
    repo: string;
    branch: string;
    filename: string;
};
/**
 * Generates a raw link to a Git file.
 *
 * @param {Props} props - The function parameters.
 * @param {string} props.branch - The branch name.
 * @param {string} props.filename - The name of the file.
 * @param {string} props.repo - The repository name.
 * @param {string} props.user - The user name.
 * @param {string} [props.gitHost] - The Git host name.
 * @return {string} The generated raw link.
 */
declare const createGitRawLink: ({ branch, filename, repo, user, gitHost: gh, }: Props) => string;
/**
 * Fetches a raw file from a GitHub repository.
 *
 * @param {Props | string} filePath - The path to the file. Either a string
 *                                    URL or an object of type Props.
 * @param {boolean} [parse=false] - Whether to parse the response as JSON.
 * @return {Promise<string | object>} - A Promise that resolves to the
 *                                      fetched file as a string. If parse
 *                                      is true, the file is parsed as JSON.
 * @throws {Error} If the fetch fails or the response is not OK.
 */
declare const getGitRawFile: (filePath: Props | string, parse?: boolean) => Promise<string | object>;

export { bytesToMb, createDate, createDateFromNowWithOffset, createGitRawLink, downloadFileFromUrl, epochToDate, fsDeleteDirectory, fsDeleteFile, getFile as fsGetFile, readFile as fsReadFile, readJSONFile as fsReadJSONFile, writeJsonToFile as fsWriteJsonToFile, writeToFile as fsWriteToFile, genOpensslRand, generateRandomNumberBetween, getGitRawFile, isValidIpAddress, isValidJson, isValidUrl, randomStringOfLength };
